#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" pyCKWConstants.py
    Define constants for the wrapper.
    In case some values need to be add/removed/updated this is a central point to perform changes.

    Author: Jeremy Diaz
    Year:   2022
"""

# The wrapper name and version
WRAPPER_NAME = 'pyCKW'
WRAPPER_VERSION = '0.2'

# Constants for Data resolutions and their respective paths
RESOLUTIONS = {
    'year' : '/year',
    'month' : '/month',
    'week' : '/week',
    'day' : '/day',
    'hour' : '/hour',
    'minute' : '/minute'
}

# Translation for datapoint items in English
# Also we define here the datapoint items we're only interested in.
# Not included as of today are:
#   anzahl_linien_fb
#   anzahl_linien_fw
#   anzahl_linien_p

DATA_TRANSLATIONS = {
    'betrag_blind_ht': 'qty_reactive_peak',
    'betrag_blind_nt': 'qty_reactive_offpeak',
    'menge_fakturiert_blind_ht': 'qty_reactive_invoiced_peak',
    'menge_fakturiert_blind_nt': 'qty_reactive_invoiced_offpeak',
    'max_leistung_faktura': 'max_power',
    'max_leistung_physisch': 'max_physical_power',
    'menge_fakturiert_ht': 'qty_invoiced_peak',
    'menge_fakturiert_nt': 'qty_invoiced_offpeak',
    'menge_physikalisch': 'qty_measured', 
    'zeitstempel_bis_utc': 'end_date',
    'zeitstempel_von_utc': 'start_date',
}

# The date format as accepted by the myCKW API
FORMAT_DATES = '%Y%m%d'

## User-configurable defaults via configurable values
# Connection Defaults
DEFAULT_HOST = "https://etility.ckw.ch"
DEFAULT_SMARTMETER_PATH = "/etility/proxy/ckw/serviceAzureDA/smartmeter/zp"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" pyCKWConfig.py
    Class file for pyCKWConfig.

    Author: Jeremy Diaz
    Year:   2022
"""

# Python Standard Library general imports
import logging

# pyCKW contants imports
from pyckw.pyCKWConstants import (
    DEFAULT_HOST,
    DEFAULT_SMARTMETER_PATH,
)

class pyCKWConfig(object):
    def __init__(self, pyCKW, meter_point, client_number, **kwargs):
        """The constructor.
        Args:
            kwargs (kwargs): Configuration options.
        """
        self.pyCKW = pyCKW
        self.config = kwargs
        self.config['meter_point'] = meter_point
        self.config['client_number'] = client_number
        self.pyCKW.info("Config loaded.")

    def dump_config(self):
        return {
            'host':             self.host,
            'smartmeter_path':  self.smartmeter_path,
            'meter_point':      self.meter_point,
            'client_number':    self.client_number
        }

    @property
    def host(self):
        return self.config.get("host", DEFAULT_HOST)
    
    @property
    def smartmeter_path(self):
        return self.config.get("smartmeter_path", DEFAULT_SMARTMETER_PATH)

    @property
    def meter_point(self):
        return self.config.get("meter_point")
    
    @property
    def client_number(self):
        return self.config.get("client_number")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" pyCKWExceptions.py
    Defines custom exception handlers.

    Author: Jeremy Diaz
    Year:   2022
"""

class pyCKWValidationError(Exception):
    """ We need a cusotm Exception handler to manage Input Validation Errors """
    def __init__(self, msg):
        self.msg = msg



from distutils.core import setup
setup(
  name = 'pyCKW',
  packages = ['pyCKW'],
  version = '0.2-1',
  license='MIT',
  description = 'Access your myCKW smartmeter data.',
  author = 'Jeremy Diaz',
  author_email = 'jd@diaznet.fr',
  url = 'https://github.com/diaznet/pyCKW',
  download_url = 'https://github.com/diaznet/pyCKW/archive/pyCKW-0.2.tar.gz',
  keywords = ['myCKW', 'smartmeter', 'data', 'power', 'consumption'],
  install_requires=[
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.7',
  ],
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" pyCKW.py
    Main class file for the pyCKW wrapper.

    Author: Jeremy Diaz
    Year:   2022
"""

# Python Standard Library general imports
import logging
import requests

# Python Standard Library specific imports
from datetime import datetime

# pyCKW class imports
from pyckw.pyCKWConfig import pyCKWConfig
from pyckw.pyCKWExceptions import pyCKWValidationError

# pyCKW contants imports
from pyckw.pyCKWConstants import (
    WRAPPER_NAME,
    WRAPPER_VERSION,
    RESOLUTIONS,
    DATA_TRANSLATIONS,
    FORMAT_DATES
)

__version__ = WRAPPER_VERSION
logger = logging.getLogger(WRAPPER_NAME)

class pyCKW(object):
    """Constructor"""
    def __init__(self, meter_point, client_number, **kwargs):
        self.info(f"{WRAPPER_NAME} {__version__} started.")
        self._config = pyCKWConfig(self, meter_point, client_number, **kwargs)
        if self.effective_level == logging.DEBUG:
            self.debug('Loaded Config:')
            for item in self._config.dump_config().items():
                self.debug(item)

    def get_consumption(self, resolution, start_date, end_date):
        # Check resolution validity
        if not RESOLUTIONS.get(resolution, None): raise pyCKWValidationError('Resolution value is not valid. Use {}.'.format(', '.join(RESOLUTIONS.keys())))
        self.debug('Resolution value \'{}\' is valid.'.format(resolution))

        # Check interval validity
        if not (start_date.strftime(FORMAT_DATES)): raise pyCKWValidationError('start_date is not valid. Use {}'.format(FORMAT_DATES))
        if not (end_date.strftime(FORMAT_DATES)): raise pyCKWValidationError('start_date is not valid. Use {}'.format(FORMAT_DATES))
        self.debug('Interval values start_date:\'{}\' and end_date:\'{}\' are valid.'.format(start_date.strftime(FORMAT_DATES), end_date.strftime(FORMAT_DATES)))

        response = requests.get(
            url = self.full_path + '/' + start_date.strftime('%Y%m%d') + '/' + end_date.strftime('%Y%m%d') + RESOLUTIONS.get(resolution)
        )
        response.raise_for_status()

        # We get in return a list of dict object parsed from the json response.
        # We will keep only the keys we need and translate them into english.
        consumption_data = []
        for consumption in response.json():
            consumption_data_point = {}
            for data_point in DATA_TRANSLATIONS:
                consumption_data_point[DATA_TRANSLATIONS[data_point]] = consumption[data_point]
            consumption_data.append(consumption_data_point)
        return(consumption_data)

    @property
    def config(self):
        """ Return loaded config """
        return self._config

    @property
    def full_path(self):
        """ The full path to the configured API """
        return self.config.host + self.config.smartmeter_path + '/' + self.config.client_number + '/' + self.config.meter_point

    def available_resolutions():
        """ Returns valid resolutions """
        return list(RESOLUTIONS.keys())

    def valid_format_dates():
        """ Returns valid date format"""
        return FORMAT_DATES

    def warning(self, msg):
        """ WARNING logging level"""    
        logger.warning(msg)

    def info(self, msg):
        """ INFO logging level"""
        logger.info(msg)

    def debug(self, msg):
        """ DEBUG logging level"""
        logger.debug(msg)

    def name_version():
        return WRAPPER_NAME + ' ' + __version__

    @property
    def effective_level(self):
        """ Return the effective logging level """
        return logger.getEffectiveLevel()
